/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "BCTable.h"

extern const SklSolverBase* SolvObj;
extern SklParaManager* ParaMngr;

BCTable::BCTable()
{
}

BCTable::~BCTable()
{
}

bool
BCTable::ParseBCTable(MaterialTable *mt)
{
  if( !ParseOuterVBC(mt) ) return false;
  if( !ParseOuterPBC(mt) ) return false;

  return true;
}

bool
BCTable::ParseOuterVBC(MaterialTable *mt)
{
  const CfgElem *bctbl, *v_elm, *bc_elm;

  const SklSolverConfig* solvCfg = NULL;
  if( !SolvObj || !(solvCfg = SolvObj->GetConfig()) ) {
    stmpd_printf("Can't get solver config object.\n");
    return false;
  }

  if( !(bctbl = solvCfg->GetTop(OUTERBND)) ){
    stmpd_printf("XML parse error : failed to get [OuterBoundary] sub element from xml config\n" );
    return false;
  }

  if( !(v_elm = bctbl->GetElemFirst("Velocity")) ){
    stmpd_printf("XML parse error : failed to get [Velocity] elelment in [OuterBoundary] sub element from xml config\n" );
    return false;
  }

  bc_elm = v_elm->GetElemFirst();
  while(bc_elm){
    BCvel *bc = new BCvel();
    if( !bc->ParseBC(bc_elm,mt) ) return false;
    m_OuterVBC.push_back(bc);
    bc_elm = v_elm->GetElemNext(bc_elm);
  }

  return true;
}

bool
BCTable::ParseOuterPBC(MaterialTable *mt)
{
  const CfgElem *bctbl, *p_elm, *bc_elm;

  const SklSolverConfig* solvCfg = NULL;
  if( !SolvObj || !(solvCfg = SolvObj->GetConfig()) ) {
    stmpd_printf("Can't get solver config object.\n");
    return false;
  }

  if( !(bctbl = solvCfg->GetTop(OUTERBND)) ){
    stmpd_printf("XML parse error : failed to get [OuterBoundary] sub element from xml config\n" );
    return false;
  }

  if( !(p_elm = bctbl->GetElemFirst("Pressure")) ){
    stmpd_printf("XML parse error : failed to get [Pressure] elelment in [OuterBoundary] sub element from xml config\n" );
    return false;
  }

  bc_elm = p_elm->GetElemFirst();
  while(bc_elm){
    BCp *bc = new BCp();
    if( !bc->ParseBC(bc_elm,mt) ) return false;
    m_OuterPBC.push_back(bc);
    bc_elm = p_elm->GetElemNext(bc_elm);
  }

  return true;
}

bool BCTable::print(FILE *fp)
{
  if( !fp ) return false;

  int i;
  fprintf( fp, "  number of OuterBC     : %d\n", m_OuterPBC.size()+m_OuterVBC.size() );
  for( i=0;i<m_OuterVBC.size();i++ ){
    if( !m_OuterVBC[i]->print(fp) ) return false;
  }
  for( i=0;i<m_OuterPBC.size();i++ ){
    if( !m_OuterPBC[i]->print(fp) ) return false;
  }

  return true;
}

