/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "Material.h"

extern const SklSolverBase* SolvObj;

Material::Material()
{
  int i;

  m_name[0] = '\0';
  m_id = -1;
  m_type = NULL;
  m_svx = NULL;
  for( i=0;i<D_MAX_MAT_PARAM;i++ ){
     m_param[i] = 0.0;
     m_setp[i] = false;
  }
}

Material::~Material()
{
}

bool
Material::ParseMat( const CfgElem *mat_elem )
{
  const char *name = mat_elem->GetName();
  if( !name ) return false;
  strcpy( m_name, name );

  if( mat_elem->isSetID() ){
    m_id = mat_elem->GetID();
  }

  const CfgParam *param = NULL;

  //type
  param = mat_elem->GetParamFirst("type");
  if( !param ){
    stmpd_printf("XML parse error : <type> is undefined in material [%s]\n", m_name );
    return false;
  }
  if( !param->GetData(&m_type) ) return false;
  if( !m_type ){
    stmpd_printf("XML parse error : failed to get <type> in material [%s]\n", m_name );
    return false;
  }

  //InFileAttr
  param = mat_elem->GetParamFirst("InFileAttr");
  if( !param ){
    stmpd_printf("XML parse error : <InFileAttr> is undefined in material [%s]\n", m_name );
    return false;
  }
  if( !param->GetData(&m_svx) ) return false;
  if( !m_svx ){
    stmpd_printf("XML parse error : failed to get <InFileAttr> in material [%s]\n", m_name );
    return false;
  }

  //parameters
  char pn[100];
  param = mat_elem->GetParamFirst();
  while( param )
  {
    const char *pname=param->GetName();
    strcpy(pn,pname);
    char *pnm = pn;
    while( pnm[0]==' ' ){
      pnm++;
    }
    if( !strncasecmp(pnm,"param",5) ){
      pnm+=5;
      int idx = atoi(pnm);
      if( idx < 0 || idx > D_MAX_MAT_PARAM ){
        stmpd_printf( "XML parse error : param index is out of range 1-%d in material [%s]\n"
                    , D_MAX_MAT_PARAM, m_name );
        return false;
      }
      float dvalue = 0;
      if( !param->GetData(&dvalue) ){
        stmpd_printf("XML parse error : failed to get <%s> [%s]\n", pn, m_name );
        return false;
      }
      m_param[idx-1] = dvalue;
      m_setp[idx-1] = true;
    }
    param = mat_elem->GetParamNext(param);
  }

  return true;
}

bool Material::print(FILE *fp)
{
  if( !fp ) return false;

  fprintf(fp,"  **************************************************************\n" );
  fprintf(fp,"  material name           : %s\n", m_name );
  fprintf(fp,"  material ID             : %d\n", m_id );
  fprintf(fp,"  material type           : %s\n", m_type );
  fprintf(fp,"  connected SVX attr      : %s\n", m_svx );
  for( int j=0;j<D_MAX_MAT_PARAM;j++ ){
    if( m_setp[j] ){
      fprintf(fp,"  param%-2d                 : %e\n", j+1, m_param[j] );
    }
  }

  return true;
}

bool Material::ReadSVX( SklParaManager* ParaMngr, SklScalar3D<SKL_REAL> *dc_phi,
                        int ni,int nj,int nk,int vc,int procGrp )
{
  int i,j,k;
  const SklCfgInFile* inFile = SolvObj->SklCfgGetInFileFirst();
  while( inFile ){
    const char *attr, *format, *fname;
    if( !(inFile->GetData(&attr, &format, &fname)) ){
      return false;
    }
    if( !strcasecmp( attr, m_svx) ){
    if( !strcasecmp(format,"sph") || !strcasecmp(format,"svx") ){
      SklVoxDataSet* pSVX = NULL;
     if( !( pSVX = ((SklSolverBase*)SolvObj)->LoadFile( ParaMngr, attr,
                                                        (unsigned)vc, procGrp ) ) ){
       stmpd_printf("file read err : %s file[%s]\n", format, fname );
        return false;
      }

      SklScalar3D<float>* pSrcF = NULL;
      SklScalar3D<double>* pSrcD = NULL;
      if( !(pSrcF = dynamic_cast<SklScalar3D<float>*>(pSVX->GetData(0))) ){
        if( !(pSrcD = dynamic_cast<SklScalar3D<double>*>(pSVX->GetData(0))) ){
          stmpd_printf("file read err :vector %s file [%s]\n",
                        format,fname );
          return false;
        }
      }
      if( pSrcF ){
        // set data
        for(k=0-vc;k<nk+vc;k++){
        for(j=0-vc;j<nj+vc;j++){
        for(i=0-vc;i<ni+vc;i++){
        dc_phi->val(i,j,k) = pSrcF->val(i,j,k);
        }}}
        delete pSrcF;

      }else if( pSrcD ){
        //set data
        for(k=0-vc;k<nk+vc;k++){
        for(j=0-vc;j<nj+vc;j++){
        for(i=0-vc;i<ni+vc;i++){
        dc_phi->val(i,j,k) = pSrcD->val(i,j,k);
        }}}
        delete pSrcD;
      }
      return true;
    }
    }
     inFile = SolvObj->SklCfgGetInFileNext( inFile );

  }
  return false;
}

