/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "MaterialTable.h"

extern const SklSolverBase* SolvObj;

MaterialTable::MaterialTable()
{
}

MaterialTable::~MaterialTable()
{
}

bool
MaterialTable::ParseMatTable()
{
  const CfgElem *mtltbl, *mat_elm;

  const SklSolverConfig* solvCfg = NULL;
  if( !SolvObj || !(solvCfg = SolvObj->GetConfig()) ) {
    stmpd_printf("Can't get solver config object.\n");
    return false;
  }

  if( !(mtltbl = solvCfg->GetTop(MDMTBL)) ){
    stmpd_printf("XML parse error : failed to get [Material_Table] sub element from xml config\n" );
    return false;
  }

  mat_elm = mtltbl->GetElemFirst();

  while(mat_elm){
    Material *mat = new Material();
    if( !mat->ParseMat(mat_elm) ) return false;
    m_material.push_back(mat);
    mat_elm = mtltbl->GetElemNext(mat_elm);
  }

  return true;
}

Material* MaterialTable::Find(const char *mat_name)
{
  Material *m = NULL;

  for( int i=0;i<m_material.size();i++ ){
      if( !strcasecmp(m_material[i]->m_name,mat_name) ){
        return m_material[i];
      }
  }


  return m;
}

bool MaterialTable::print(FILE *fp)
{
  if( !fp ) return false;

  fprintf( fp, "  number of material    : %d\n", m_material.size() );
  for( int i=0;i<m_material.size();i++ ){
    if( !m_material[i]->print(fp) ) return false;
  }

  return true;
}

bool MaterialTable::ReadSVX( SklParaManager* ParaMngr,SklScalar3D<SKL_REAL> *dc_phi,
                             int ni, int nj, int nk, int vc, int procGrp )
{
    int i;
    bool bSVX = false;
    for( i=0;i<m_material.size();i++ ){
      Material *mat = m_material[i];
      if( !strcasecmp(m_material[i]->m_type,"Solid") ){
        bSVX = mat->ReadSVX(ParaMngr,dc_phi,ni,nj,nk,vc,procGrp);
        break;
      }
    }
  if( bSVX ){ 
    return true;
  }else{
    return false;
  }
}
