#ifndef _SPHERE_SOLVER_PARTICLE_H_
#define _SPHERE_SOLVER_PARTICLE_H_

#include "SklSolverBase.h"
#include "SklUtil.h"

class Particle
{
public:
  Particle(SklSolverBase* solvObj, SklParaManager* paraMngr);
  virtual ~Particle();

public:  
  // Solver class
  SklSolverBase *m_solvObj;

  // parallel manager
  SklParaManager* m_paraMngr;

  // MPI node No
  int id;

  // input file name
  const char *ifname;

  // base name of output file
  const char *ofbase;

  // output interval
  int interval;

  // number of particle
  int ng;

  // particle coordinate
  SKL_REAL *xgo,*ygo,*zgo;  //initial
  SKL_REAL *xg,*yg,*zg;
  SKL_REAL *xgn,*ygn,*zgn;  //new

  // particle velocity
  SKL_REAL *ug,*vg,*wg;

  // Node No. of each particle (MPI Node ID + 1)
  int *nid;

  // comunication buffer
  SKL_REAL *rbuf;
  int *ibuf;

protected:
  bool ReadScat(const char* fname);
  bool ReadScatFile(const char* fname);
  bool Allocate();
  bool SetNodeID();

  bool WriteFile(const char* base, int step, SKL_REAL time);
  bool UpdateNodeID();
  bool AllreduceSUM(SKL_REAL *data);
  bool AllreduceSUM(int *data);

public:
  bool Init();
  bool IsActive();
  bool WriteFile(unsigned step, SKL_REAL time, bool forced_out=false);
  bool CommParticle();


};

#endif /*_SPHERE_SOLVER_PARTICLE_H_*/

