/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "SklSolverSPH3D.h"
#include "SklUtil.h"

int SklSolverSPH3D::Allocate( size_t& total_mem )
{
  total_mem = 0;
 
  size_t n=0;
  unsigned int dims[4];
  dims[0] = ni;
  dims[1] = nj;
  dims[2] = nk;
  dims[3] = 6;
  unsigned vcell = vc;

  // Vector3DEx
  n=(ni+2*vc)*(nj+2*vc)*(nk+2*vc)*3;
  if( !SklIsCheckMode() ){ 
    if( !(dc_uvw = dynamic_cast<SklVector3DEx<SKL_REAL>*>(
                     SklAllocateArray(ParaMngr,
                                      "uvw",
                                      SKL_CLASS_VECTOR3DEX,
                                      SKL_ARRAY_DTYPE_REAL,
                                      dims,
                                      vcell,
                                      procGrp))) ) return -1;

    if( !(dc_uvwn = dynamic_cast<SklVector3DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "uvwn",
                                       SKL_CLASS_VECTOR3DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_uvwo = dynamic_cast<SklVector3DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "uvwo",
                                       SKL_CLASS_VECTOR3DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_outwv= dynamic_cast<SklVector3DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "outwv",
                                       SKL_CLASS_VECTOR3DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_gravity = dynamic_cast<SklVector3DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "gravity",
                                       SKL_CLASS_VECTOR3DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

     if( !(uvw=dc_uvw->GetData()) ) return -1;
     if( !(uvwn=dc_uvwn->GetData()) ) return -1;
     if( !(uvwo=dc_uvwo->GetData()) ) return -1;
     if( !(outwv=dc_outwv->GetData()) ) return -1;
     if( !(gravity=dc_gravity->GetData()) ) return -1;
     if( !SklInitializeSKL_REAL(uvw,0.0,n) ) return -1;
     if( !SklInitializeSKL_REAL(uvwn,0.0,n) ) return -1;
     if( !SklInitializeSKL_REAL(uvwo,0.0,n) ) return -1;
     if( !SklInitializeSKL_REAL(gravity,0.0,n) ) return -1; 
     if( !SklInitializeSKL_REAL(outwv,0.0,n) ) return -1;
  }
  total_mem += n*sizeof(SKL_REAL)*5;

  // Scalar3D
  n=(ni+2*vc)*(nj+2*vc)*(nk+2*vc);
  if( !SklIsCheckMode() ){ 
    if( !(dc_p    = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "p",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_div  = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "div",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_phi  = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "phi",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_phin = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "phin",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_phio = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "phio",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_pk   = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "pk",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_advsx = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "advsx",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_advsy = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "advsy",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_advsz = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "advsz",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;


    if( !(dc_outw = dynamic_cast<SklScalar3D<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "outw",
                                       SKL_CLASS_SCALAR3D,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(p=dc_p->GetData()) ) return -1;
    if( !(div=dc_div->GetData()) ) return -1;
    if( !(phi=dc_phi->GetData()) ) return -1;
    if( !(phin=dc_phin->GetData()) ) return -1;
    if( !(phio=dc_phio->GetData()) ) return -1;
    if( !(pk=dc_pk->GetData()) ) return -1;
    if( !(advsx=dc_advsx->GetData()) ) return -1;
    if( !(advsy=dc_advsy->GetData()) ) return -1;
    if( !(advsz=dc_advsz->GetData()) ) return -1;
    if( !(outw=dc_outw->GetData()) ) return -1;
    if( !SklInitializeSKL_REAL(p,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(div,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(phi,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(phin,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(phio,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(pk,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(advsx,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(advsy,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(advsz,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(outw,0.0,n) ) return -1;
  }
  total_mem += n*sizeof(SKL_REAL)*10;

  // Scalar4DEx
  n=(ni+2*vc)*(nj+2*vc)*(nk+2*vc)*6;
  if( !SklIsCheckMode() ){ 
    if( !(dc_sg   = dynamic_cast<SklScalar4DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "sg",
                                       SKL_CLASS_SCALAR4DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_sgn  = dynamic_cast<SklScalar4DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "sgn",
                                       SKL_CLASS_SCALAR4DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_sgo  = dynamic_cast<SklScalar4DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "sgo",
                                       SKL_CLASS_SCALAR4DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_bl   = dynamic_cast<SklScalar4DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "bl",
                                       SKL_CLASS_SCALAR4DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_bln  = dynamic_cast<SklScalar4DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "bln",
                                       SKL_CLASS_SCALAR4DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(dc_blo  = dynamic_cast<SklScalar4DEx<SKL_REAL>*>(
                      SklAllocateArray(ParaMngr,
                                       "blo",
                                       SKL_CLASS_SCALAR4DEX,
                                       SKL_ARRAY_DTYPE_REAL,
                                       dims,
                                       vcell,
                                       procGrp))) ) return -1;

    if( !(sg=dc_sg->GetData()) ) return -1;
    if( !(sgn=dc_sgn->GetData()) ) return -1;
    if( !(sgo=dc_sgo->GetData()) ) return -1;
    if( !(bl=dc_bl->GetData()) ) return -1;
    if( !(bln=dc_bln->GetData()) ) return -1;
    if( !(blo=dc_blo->GetData()) ) return -1;
    if( !SklInitializeSKL_REAL(sg,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(sgn,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(sgo,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(bl,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(bln,0.0,n) ) return -1;
    if( !SklInitializeSKL_REAL(blo,0.0,n) ) return -1;
  }
  total_mem += n*sizeof(SKL_REAL)*6;

/**** work array ****/

  nw  = 0;   //final size
  int nws = 0;    //subroutine size

  /** mkpost work size **/
  p1=0;                     // x4
  p2=p1+(ni+1)*(nj+1)*nk;   // y4
  p3=p2+(ni+1)*(nj+1)*nk;   // x4
  p4=p3+(ni+1)*(nj+1)*nk;   // f4
  p5=p4+(ni+1)*(nj+1)*nk;   // p4
  p6=p5+(ni+1)*(nj+1)*nk;   //uvw4
  nw=p6+3*(ni+1)*(nj+1)*(nk+1);    

  /**  solu,solp work size **/

  nws=1;
  nws+=1;
  nw=max(nw,nws);   // total work size
  if( !SklIsCheckMode() ){ 
    if( !(work = SklAllocateSKL_REAL(nw,true)) ) return -1;
  }
  total_mem += nw*sizeof(SKL_REAL);


  // communication buffer for 4-color SOR
  buffsz[0] = 1;
  buffsz[1] = 1;
  buffsz[2] = 1;
  if( ParaMngr->IsParallel() && isor==SOR4 ){
    buffsz[0] = ((nj+1)/2) * ((nk+1)/2);
    buffsz[1] = ((ni+1)/2) * ((nk+1)/2);
    buffsz[2] = ((ni+1)/2) * ((nj+1)/2);
  }
  if( !SklIsCheckMode() ){ 
    if( !(recvib = SklAllocateSKL_REAL(buffsz[0]*2, true)) ) return -1;
    if( !(sendib = SklAllocateSKL_REAL(buffsz[0]*2, true)) ) return -1;
    if( !(recvjb = SklAllocateSKL_REAL(buffsz[1]*2, true)) ) return -1;
    if( !(sendjb = SklAllocateSKL_REAL(buffsz[1]*2, true)) ) return -1;
    if( !(recvkb = SklAllocateSKL_REAL(buffsz[2]*2, true)) ) return -1;
    if( !(sendkb = SklAllocateSKL_REAL(buffsz[2]*2, true)) ) return -1;
  }

  total_mem += sizeof(SKL_REAL)*buffsz[0]*4;
  total_mem += sizeof(SKL_REAL)*buffsz[1]*4;
  total_mem += sizeof(SKL_REAL)*buffsz[2]*4;

  return 1;

}

