/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "SklSolverSPH3D.h"
#include "SklUtil.h"

int SklSolverSPH3D::InitOutputFile()
{
  int i;

  unsigned int usz[3] = {ni,nj,nk};

  const SklCfgOutFile* outFile = SklCfgGetOutFileFirst();
  while( outFile ){
    const char *format, *attr;
    if( !(format = outFile->GetFormat()) ) {
      stmpd_printf("\tParsing error : OutFile description\n");

      return -1;
    }
    if( !(attr=outFile->GetAttr()) ){
      stmpd_printf("\tParsing error : OutFile description\n");

      return -1;
    }
    if ( !strcasecmp(format, "sph") ){
      if( !strcasecmp(attr,"Velocity")){
        if( !(outVel=InitFile(outFile,usz,org,pch,dc_uvw)) ){
          stmpd_printf("\tInit file[Velocity] failed.\n");

          return -1;
        }
      }
      else if( !strcasecmp(attr,"Pressure")){
        if( !(outPres=InitFile(outFile,usz,org,pch,dc_p)) ){
          stmpd_printf("\tInit file[Pressure] failed.\n");

          return -1;
        }
      }
      else if( !strcasecmp(attr,"Phi")){
        if( !(outPhi=InitFile(outFile,usz,org,pch,dc_phi)) ){
          stmpd_printf("\tInit file[Phi] failed.\n");

          return -1;
        }
      }
      else if( !strcasecmp(attr,"MisesStress")){
        if( !(outMis=InitFile(outFile,usz,org,pch,dc_outw)) ){
          stmpd_printf("\tInit file[MisesStress] failed.\n");

          return -1;
        }
      }
      else if( !strcasecmp(attr,"VelocityGradient")){
        if( !(outVG=InitFileDiv(outFile,usz,org,pch,dc_outw,0)) ){
          stmpd_printf("\tInit file[VelocityGradient] failed.\n");

          return -1;
        }

      }
      else if( !strcasecmp(attr,"RestartFile")){
        if( !(outRes=InitFileDiv(outFile,usz,org,pch,dc_outwv,1)) ){
          stmpd_printf("\tInit file[RestartFile] failed.\n");

          return -1;
        }
      }
    }
    outFile=SklCfgGetOutFileNext(outFile);
  }

  return 1;
}

SklVoxDataSet** SklSolverSPH3D::InitFileDiv(const SklCfgOutFile* cfg,
                                            const unsigned int size[3],
                                            const SKL_REAL origin[3],
                                            const SKL_REAL pitch[3],
                                            SklArayBase* array,
                                            const unsigned int flag)
{
  int i;

  int format = SklVoxDataSet::SPH_FMT;

  const char* basename = cfg->GetBaseName();
  size_t len = strlen(basename)+20;
  char *bn = new char[len];
  int itrvl = cfg->GetInterval();
  bool mio = cfg->IsMultiOutput();

  unsigned long long lsz[3];
  double dorg[3];
  double dpch[3];

  for(i=0;i<3;i++){
  lsz[i]=(unsigned long long)size[i];
  dorg[i]=(double)origin[i];
  dpch[i]=(double)pitch[i];
  }

  const char comp_VG[9][5]   ={"dudx","dvdy","dwdz"
                              ,"dudy","dvdx","dudz"
                              ,"dwdx","dvdz","dwdy"};
  const char comp_RT[11][11] ={"uvw","uvwo","p_phi_phio"
                              ,"sg123" ,"sg456","sgo123"
                              ,"sgo456","bl123","bl456"
                              ,"blo123","blo456"};
  int ndata = 11;
  SklVoxDataSet** vox = NULL;
  const char* comp_name[11];
  if( flag==0 ){
    ndata = 9;
    for( i=0;i<ndata;i++ ){
      comp_name[i] = comp_VG[i];
    }
  } else {
    ndata = 11;
    for( i=0;i<ndata;i++ ){
      comp_name[i] = comp_RT[i];
    }
  }

  vox = new SklVoxDataSet*[ndata];
  for( i=0;i<ndata;i++ ){
    // create SklVoxDataSet
    if( !(vox[i] = CreateFileData(format, lsz, dorg, dpch, array )) ) return NULL;

    // set basename
    sprintf( bn, "%s_%s_", basename,comp_name[i] );

    // Init
    if( !m_fileCntl.Init(bn, itrvl, mio, NULL, NULL, vox[i]) ){
      return NULL;
    }
  }

  return vox;
}

int SklSolverSPH3D::FileOutput(int step, SKL_REAL time, unsigned mode)
{
  bool force=false;
  if( mode==FORCEDOUT ) force = true;

  unsigned ustep = step;

  if( outVel ){
    if( !WriteFile(outVel, ustep, time, 0, force, vc_out) ){
      stmpd_printf( "\twrite Velocity false\n");

      return -1;
    }
  }
  if( outPres ){
    if( !WriteFile(outPres, ustep, time, 0, force, vc_out) ) {
      stmpd_printf( "\twrite Pressure false\n");

      return -1;
    }
  }
  if( outPhi ){
    if( !WriteFile(outPhi, ustep, time, 0, force, vc_out) ) {
      stmpd_printf( "\twrite Phi false\n");

      return -1;
    }
  }
  if( outMis ){
    if( CheckFileOut(outMis,ustep,force) ){
      calmises_(outw,sg,&ni,&nj,&nk,&vc);
      if( !WriteFile(outMis, ustep, time, 0, force, vc_out) ) {
        stmpd_printf( "\twrite MisesStress false\n");

        return -1;
      }
    }
  }
  if( outVG ){
    if( CheckFileOut(outVG[0],ustep,force) ){
      int flag;
      for(flag=1;flag<10;flag++){

        if( outVG[flag-1] ){
          cal_vg_(&flag,&ni,&nj,&nk,&vc,&dx,&dy,&dz,uvw,outw);
          if( !WriteFile(outVG[flag-1], ustep, time, 0, force, vc_out) ) {
            stmpd_printf( "\twrite VelocityGradient false\n");
            return -1;
          }
        }
      }
    }
  }
  if( outRes ){

    if( CheckFileOut(outRes[0],ustep,force) && ustep != 0 ){

      int i;
      for(i=0;i<11;i++){

        if( outRes[i] ){
          if( i==0 ){
          restart_uvw_(outwv,uvw,&vc,&ni,&nj,&nk);
          }else if( i==1 ){
          restart_uvw_(outwv,uvwo,&vc,&ni,&nj,&nk);
          }else if( i==2 ){
          restart_p_(outwv,p,phi,phio,&vc,&ni,&nj,&nk);
          }else if( i==3 ){
          int flag=1;
          restart_sgbl_(outwv,sg,&vc,&ni,&nj,&nk,&flag);
          }else if( i==4 ){
          int flag=2;
          restart_sgbl_(outwv,sg,&vc,&ni,&nj,&nk,&flag);
          }else if( i==5 ){
          int flag=1;
          restart_sgbl_(outwv,sgo,&vc,&ni,&nj,&nk,&flag);
          }else if( i==6 ){
          int flag=2;
          restart_sgbl_(outwv,sgo,&vc,&ni,&nj,&nk,&flag);
          }else if( i==7 ){
          int flag=1;
          restart_sgbl_(outwv,bl,&vc,&ni,&nj,&nk,&flag);
          }else if( i==8 ){
          int flag=2;
          restart_sgbl_(outwv,bl,&vc,&ni,&nj,&nk,&flag);
          }else if( i==9 ){
          int flag=1;
          restart_sgbl_(outwv,blo,&vc,&ni,&nj,&nk,&flag);
          }else if( i==10 ){
          int flag=2;
          restart_sgbl_(outwv,blo,&vc,&ni,&nj,&nk,&flag);
          }

          if( !WriteFile(outRes[i], ustep, time, 0, force, vc) ) {
            stmpd_printf( "\twrite RestartFile false\n");
            return -1;
          }
       }

      }
    }
  }

  if( pt ){
    if( pt->IsActive() ){
      pt->WriteFile(ustep,time,force);
    }
  }

  if( imkuvp>0 && step%imkuvp == 0 ){
    mkuvp_(&vc,&ni,&nj,&nk,&step,&dx,&dy,&dz,p,phi,uvw,&REAL_MIN);

    mkpost_(&vc,&ni,&nj,&nk,&step,&time,&dx,&dy,&dz,
            p,phi,uvw,sg,work+p1,work+p2,work+p3,work+p4,work+p5,work+p6);
  }


  return 1;
}

bool SklSolverSPH3D::CheckFileOut(SklVoxDataSet* dSet, unsigned step, bool force)
{
  if( force ){
    return true;
  }

    unsigned int interval = m_fileCntl.GetInterval(dSet);
    if( (step%interval)==0 ) return true;

  return false;
}

bool SklSolverSPH3D::ReadSVX( SklParaManager* ParaMngr )
{
  return MAT.ReadSVX(ParaMngr,dc_phi,ni,nj,nk,vc,procGrp);
}
