/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "SklSolverSPH3D.h"
#include "SklUtil.h"

bool SklSolverSPH3D::GetXML()
{
  if( !GetXML_Steer() ) return false;
  if( !GetXML_Mat() ) return false;
  if( !GetXML_BC() ) return false;

  return true;
}

bool SklSolverSPH3D::GetXML_Steer()
{
  const CfgElem *root, *elem;
  const CfgParam *param;

  if( !(root = SklCfgGetTop(STEER)) ){
    stmpd_printf("XML parse error : failed to get [Steer] sub element from xml config\n");
    return false;
  }

//DT
  float fdt=0.0;
  if( !(param = root->GetParamFirst("DT")) ) return false;
  if( !(param->GetData(&fdt)) ) return false;
  fixedDT = fdt;

//step
  unsigned int step = SklCfgGetCalcSteps();


// SOR
  const char* cpe=NULL;
  int iter=0;
  float beta=0.0;
  float eps=0.0;

  if( !(param = root->GetParamFirst("PoissonEquation")) ) return false;
  if( !(param->GetData(&cpe)) ) return false;
  if( !strcasecmp(cpe,"SOR" ) ){
    isor = SOR;
  }else if( !strcasecmp(cpe,"SOR4O" ) ){
    isor = SOR4O;
  }else if( !strcasecmp(cpe,"SOR4" ) ){
    isor = SOR4;
  } else {
    return false;
  }

  if( isor==SOR || isor==SOR4 || isor==SOR4O ){
    if( !(param = root->GetParamFirst("ITER")) ) return false;
    if( !(param->GetData(&iter)) ) return false;
    itersor = iter;

    if( !(param = root->GetParamFirst("BETA")) ) return false;
    if( !(param->GetData(&beta)) ) return false;
    sorbeta = beta;

    if( !(param = root->GetParamFirst("EPS")) ) return false;
    if( !(param->GetData(&eps)) ) return false;
    soreps = eps;
  }
    
// Gravity
  if( !(elem = root->GetElemFirst("Gravity")) ) return false;
  SklUtil::getVecParams(elem,"u","v","w",grvuvw);

//old format intarval
  int iof=0;
  if( (param = root->GetParamFirst("OldFormatInterval")) ){ 
    if( (param->GetData(&iof)) ){  
      if( ipara==-1 ){
        imkuvp=iof;
      }
    }
  }

//Check Err
  int cher=0;
  if( (param = root->GetParamFirst("CheckErr")) ){
    if( (param->GetData(&cher)) ){
      icherr=cher;  
    }
  }

//CN_ITER
  int icn=0;
  if( !(param = root->GetParamFirst("CN_ITER")) ) return false;
  if( !(param->GetData(&icn)) ) return false;
  imax=icn;

  return true;
}


bool SklSolverSPH3D::GetXML_Mat()
{
  return MAT.ParseMatTable();
}

bool SklSolverSPH3D::GetXML_BC()
{
  return BC.ParseBCTable(&MAT);
}
