/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "SklSolverSPH3D.h"
#include "SklUtil.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  setup for restart
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

bool SklSolverSPH3D::SetupRestart()
{
  if( SklCfgGetStartType() == Restart ){
    if( !ReadRestartFile() ) return false;
  }

  return true;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  read restart file
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

bool SklSolverSPH3D::ReadRestartFile()
{
  int i,j,k,l,n;
  int flag1=1,flag2=2;
  unsigned int usz[3] = {sz[0],sz[1],sz[2]};
  const SklCfgInFile* inFile = SklCfgGetInFileFirst();
  while( inFile ){
    const char *attr, *format, *fname;
    if( !(inFile->GetData(&attr, &format, &fname)) ){
    return false;
    }

    const char comp_name[11][11]={"uvw","uvwo","p_phi_phio"
                                 ,"sg123" ,"sg456","sgo123"
                                 ,"sgo456","bl123","bl456"
                                 ,"blo123","blo456"};

    if( !strcasecmp(attr, "RestartFile") ){
      for( n=0;n<11;n++ ){
        SklVoxDataSet *vox = LoadRestartFile(inFile,comp_name[n],usz,vc,procGrp);

        //time&step
        if( n==0 ){
          //step
          int stp = 0;
          if( !vox->GetStep(stp) ){
            long long lstp = 0;
            if( !vox->GetStep(lstp) ){
              stmpd_printf("file read err : step of %s file [%s]\n", format,fname );
              return false;
            }
            stp = lstp;
          }
          //time
          SKL_REAL tim = 0.0;
          if( !vox->GetTime(tim) ){
            stmpd_printf("file read err : time of %s file [%s]\n", format,fname );
            return false;
          }

          // set
          SklSetBaseStep(stp);
          time = tim;
        }         
 
        SklArayBase* pb = vox->GetData();
        SklVector3DEx<SKL_REAL>* pSrc = dynamic_cast<SklVector3DEx<SKL_REAL>*>
                                        (vox->GetData());


        for( l=0;l<3;l++ ){
        for( i=0-vc;i<ni+vc;i++){
        for( j=0-vc;j<nj+vc;j++){
        for( k=0-vc;k<nk+vc;k++){
          dc_outwv->val(l,i,j,k) = pSrc->val(l,i,j,k);
        }}}}


        if( n==0 ) load_restart_uvw_(outwv,uvw,&vc,&ni,&nj,&nk);
        if( n==1 ) load_restart_uvw_(outwv,uvwo,&vc,&ni,&nj,&nk);
        if( n==2 ) load_restart_p_(outwv,p,phi,phio,&vc,&ni,&nj,&nk);
        if( n==3 ) load_restart_sgbl_(outwv,sg,&vc,&ni,&nj,&nk,&flag1);
        if( n==4 ) load_restart_sgbl_(outwv,sg,&vc,&ni,&nj,&nk,&flag2);
        if( n==5 ) load_restart_sgbl_(outwv,sgo,&vc,&ni,&nj,&nk,&flag1);
        if( n==6 ) load_restart_sgbl_(outwv,sgo,&vc,&ni,&nj,&nk,&flag2);
        if( n==7 ) load_restart_sgbl_(outwv,bl,&vc,&ni,&nj,&nk,&flag1);
        if( n==8 ) load_restart_sgbl_(outwv,bl,&vc,&ni,&nj,&nk,&flag2);
        if( n==9 ) load_restart_sgbl_(outwv,blo,&vc,&ni,&nj,&nk,&flag1);
        if( n==10) load_restart_sgbl_(outwv,blo,&vc,&ni,&nj,&nk,&flag2);

        delete pSrc;
       }
     }
     inFile = SklCfgGetInFileNext( inFile );
  }
  return true;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  load restart file
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
SklVoxDataSet* SklSolverSPH3D::LoadRestartFile( const SklCfgInFile* inFile,
                                               const char* comp_name,   
                                               const unsigned int size[3],
                                               unsigned vc,
                                               int procGrp)
{
  if( !inFile ) return NULL;
  const char* fmt = inFile->GetFormat();
  if( !fmt ) return NULL;
  SklVoxDataSet::FmtType format = SklVoxDataSet::UNKNOWN_FMT;
  if( !strcasecmp( fmt, "sph") ){
    format = SklVoxDataSet::SPH_FMT;
  }
  else return NULL;

  const char* fname = inFile->GetFileName();
  bool mio = inFile->IsMultiInput();

  char rfname[512];
  memset(rfname,0x00,512);

  char *ptr = strstr(fname,".sph");
  char *pn=NULL;
  for( char *p=ptr-1;p>=fname;p-- ){
    if( p[0]>='0' && p[0]<='9' ) continue;
    if( p-3 < fname ){
    pn = p+1;
    break;
    }
    if( p[-2]=='_' &&
       (p[-1]=='i' || p[-1]=='I') &&
       (p[ 0]=='d' || p[ 0]=='D') ){
      p-=2;
      continue;
    }
    pn = p+1;
    break;
  }

  strncpy( rfname, fname, pn-fname );
  strcat( rfname, "_" );
  strcat( rfname, comp_name );
  strcat( rfname, "_" );
  strcat( rfname, pn );

  return m_fileCntl.Load(format, rfname, size, vc, ParaMngr, mio, procGrp);
}


