/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "SklSolverSPH3D.h"
#include "SklUtil.h"

int SklSolverSPH3D::SetupDomain()
{
  int i;

  if( ParaMngr->IsParallel() ){
    ipara = 1;
    ID = ParaMngr->GetMyID(procGrp);
  }

  unsigned usz[3];
  for( i=0; i<3; i++ ){
    usz[i] = 0;
  }

  if( !SklUtil::getCellInfo(3, usz, org, pch, wth) ) return -1;

  ni = sz[0] = usz[0];
  nj = sz[1] = usz[1];
  nk = sz[2] = usz[2];

  xl = wth[0];
  yl = wth[1];
  zl = wth[2];

  dx = pch[0];
  dy = pch[1];
  dz = pch[2];


  if( ParaMngr->IsParallel() ){
    int idiv,jdiv,kdiv;
    if( !GetCfgVoxelDivisionMethod(idiv,jdiv,kdiv) ){
      idiv=jdiv=kdiv= 0;
    }

    unsigned int uisz[3];
    uisz[0] = usz[0];
    uisz[1] = usz[1];
    uisz[2] = usz[2];
    SklParaComponent* ParaComp = SklGetParaComponent();
    if( ParaComp->SklVoxelInit(uisz[0],uisz[1],uisz[2],idiv,jdiv,kdiv,procGrp)!=SKL_PARACMPO_SUCCESS){
      stmpd_printf("\tID %d : Voxel Initialize error.\n", ID);
      return -1;
    }
    const unsigned int* sz_para = ParaMngr->GetVoxelSize(procGrp);
    if( !sz_para ){
      stmpd_printf("\nID %d : Can't get voxel size.\n", ID );
      return -1;
    }

    ni = sz_para[0];
    nj = sz_para[1];
    nk = sz_para[2];

    xl = SKL_REAL(ni)*dx;
    yl = SKL_REAL(nj)*dy;
    zl = SKL_REAL(nk)*dz;

    for( i=0;i<3;i++ ){
      int start_idx = ParaMngr->GetVoxelHeadIndex(ID,i,procGrp);
      org[i] += pch[i]*start_idx;
    }
  }

  // Create loop band table for fortran subroutine
  if( ParaMngr->IsParallel() ){
    nID[X_MINUS] = ParaMngr->GetCommID(-1, 0, 0, procGrp);
    nID[Y_MINUS] = ParaMngr->GetCommID( 0,-1, 0, procGrp);
    nID[Z_MINUS] = ParaMngr->GetCommID( 0, 0,-1, procGrp);
    nID[X_PLUS]  = ParaMngr->GetCommID( 1, 0, 0, procGrp);
    nID[Y_PLUS]  = ParaMngr->GetCommID( 0, 1, 0, procGrp);
    nID[Z_PLUS]  = ParaMngr->GetCommID( 0, 0, 1, procGrp);
  }

  // set loop start index for SOR4
  if( ParaMngr->IsParallel() ){
    int start_idx = 0;
    start_idx = ParaMngr->GetVoxelHeadIndex(ID,0,procGrp);
    if( start_idx%2==1 ){
      for(i=0;i<8;i++) idx_si[i] = 3 - idx_si[i];
    }
    start_idx = ParaMngr->GetVoxelHeadIndex(ID,1,procGrp);
    if( start_idx%2==1 ){
      for(i=0;i<8;i++) idx_sj[i] = 3 - idx_sj[i];
    }
    start_idx = ParaMngr->GetVoxelHeadIndex(ID,2,procGrp);
    if( start_idx%2==1 ){
      for(i=0;i<8;i++) idx_sk[i] = 3 - idx_sk[i];
    }
  }

  return 1;
}


