/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "SklSolverSPH3D.h"

int
SklSolverSPH3D::SklSolverInitialize() {
/* USER WRITE
   Please write solver initialize code.
     Return value  ->   -1 : error
                         0 : forced termination
                         1 : normal return
*/

  SklParaComponent* ParaComp = SklGetParaComponent();
  if( !(ParaMngr = ParaComp->GetParaManager()) ) return -1;

  // Domain Setup
  if( SetupDomain() !=1 ) return -1;

  // get XML param
  if( GetXML() !=1 ) return -1;

  // Memory Allocate
  size_t total_mem = 0;
  if( Allocate(total_mem) !=1 ) return -1;

  // print parameters
  PrintParam(total_mem);
  if( SklIsCheckMode() ){
    PRINT_TITLE(stdout,"solver is check mode");
    fprintf(stdout,"\n");
    return 0;
  }

  // read SVX
//  if( ReadSVX( ParaMngr) !=1 ) return -1;

  //initialize
  init_(&ipara,nID,&ID,&ni,&nj,&nk,&vc,
       &rnu00,&dx,&dy,&dz,wth,
        phi,phio,uvw,uvwo,p,sg,sgo,bl,blo,
        gravity,grvuvw,ftime);

  //Restart
  if( !SetupRestart() ){
    return -1;
  }

  dc_uvw->CommBndCell(vc);
  dc_uvwo->CommBndCell(vc);
  dc_sg->CommBndCell(vc);
  dc_sgo->CommBndCell(vc);
  dc_bl->CommBndCell(vc);
  dc_blo->CommBndCell(vc);
  dc_phi->CommBndCell(vc);
  dc_phio->CommBndCell(vc);
  dc_p->CommBndCell(vc);

  // for particle trace
  pt = new Particle(this,ParaMngr);
  if( !pt ) return -1;
  if( !pt->Init() ) return -1;
  if( pt->IsActive() ){
    initg_(&vc,&ni,&nj,&nk,&dx,&dy,&dz,
           uvw,&ID,&pt->ng,pt->nid,pt->ug,pt->vg,pt->wg,pt->xg,pt->yg,pt->zg);
  }

  // Output file setting
  if( InitOutputFile() !=1) return -1;

  // File output
  int step = 0+SklGetBaseStep();
  if( FileOutput(step,time,FORCEDOUT) !=1) return -1;

  // history setup
  SetupHistory();

  if( ID==0 ){
    PRINT_TITLE(stdout,"initialize end");
    fprintf(stdout,"\n");
  }

  return 1;
}

