/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "SklSolverSPH3D.h"

int
SklSolverSPH3D::SklSolverLoop(const unsigned int step) {
/* USER WRITE
   Please write solver main code.
     Return value  ->   -1 : error
                         0 : forced termination
                         1 : normal return
*/
  ParaMngr->Barrier(procGrp);
  double LoopPrevTime = SklGetTime();
  double btime = 0.0;

  // time step
  unsigned ustep = step + SklGetBaseStep();
  int nstep = ustep;

  // check history out
  int imon_t = 0;
  bool bMon = false;
  bool bHist = false;
  CheckHistoryOut( ustep, bMon, bHist );
  if( bMon || bHist ){
    imon_t=nstep;
  }else{
    imon_t=nstep+1;
  }

  SKL_REAL dt = fixedDT;
  SKL_REAL timeo = time;
  caldt_(&ID,&vc,&ni,&nj,&nk,&imon_t,&nstep,
         &rnu00,&gmd00,&cfl,&dt,&timeo,&dx,&dy,&dz,uvw,&REAL_MAX,rlog,ftime);
  //dt=fixedDT;
  time += dt;
  rlog[0] = dt;
  rlog[3] = time;


  solphi_(&ipara,nID,&ID,sz,&vc,&ni,&nj,&nk,&icherr,&nstep,&imon_t,
          &time,&dt,&dx,&dy,&dz,
          phi,phio,phin,uvw,uvwo,&REAL_MIN,rlog,ftime);

//add 2011/02/18 EUL s
  SKL_REAL dxinv,dyinv,dzinv;
  dxinv = 1.0/dx;
  dyinv = 1.0/dy;
  dzinv = 1.0/dz;
  solbl_(&ni,&nj,&nk,&vc,&ipara,nID,
         uvw,uvwo,bl,blo,bln,phin,ftime,
         &dt,&dxinv,&dyinv,&dzinv,&REAL_MIN);
//add 2011/02/18 EUL e
  
#if 1
  solu_(&ipara,&ID,nID,sz,&ni,&nj,&nk,&vc,&imax,&icherr,&nstep,&imon_t,
        &isor,&itersor,&sorbeta,&soreps,
        &pi,&time,&dt,&dx,&dy,&dz,&grv,&rnu00,&gmd00,&xl,&yl,&zl,
        uvw,uvwo,uvwn,p,phin,div,sg,sgn,bl,blo,bln,
        advsx,advsy,advsz,pk,work,&nw,
        idx_si,idx_sj,idx_sk,buffsz,
        recvib,recvjb,recvkb,sendib,sendjb,sendkb,
        &REAL_MIN,rlog,ilog,ftime,&itersort);
#else
  solu1_(&ipara,&ID,nID,&ni,&nj,&nk,&vc,
         &dx,&dy,&dz,&dt,&grv,
         uvw,uvwo,uvwn,p,pk,sg,sgn,
         bl,blo,bln,advsx,advsy,advsz,
         ftime,&REAL_MIN);

  solu2_(&ipara,&ID,nID,sz,&ni,&nj,&nk,&vc,
         &imax,&nstep,&icherr,&imon_t,&dx,&dy,&dz,
         &dt,&time,&rnu00,&gmd00,
         phin,uvw,uvwn,bln,sgn,advsx,advsy,advsz,
         ftime,rlog);

  solu3_(&ipara,&ID,nID,sz,&ni,&nj,&nk,&vc,&dx,&dy,&dz,
         &xl,&yl,&zl,&nstep,&time,&dt,&rnu00,&pi,
         &isor,&itersor,&sorbeta,&soreps,
         idx_si,idx_sj,idx_sk,buffsz,
         recvib,recvjb,recvkb,sendib,sendjb,sendkb,
         div,p,pk,phin,uvwn,sgn,work,&nw,
         &icherr,&imon_t,&itersort,rlog,ilog,ftime,
         &REAL_MIN);
#endif


  if( pt ){
    if( pt->IsActive() ){
      solxg_(&vc,&ni,&nj,&nk,&dt,&dx,&dy,&dz,wth,uvwn,
             pt->xg,pt->yg,pt->zg,pt->xgn,pt->ygn,pt->zgn,
             pt->ug,pt->vg,pt->wg,&pt->ng,pt->nid,&ID);
      pt->CommParticle();
    }
  }


  if( bMon || bHist ){
    budget_(&ipara,&ID,nID,sz,&ni,&nj,&nk,&vc,&time,&dx,&dy,&dz,&dt,wth,
            sg,sgn,uvw,uvwn,phi,phin,rlog,&grv);
  }


  tadv_(&vc,&ni,&nj,&nk,uvw,uvwn,uvwo,phi,phin,phio,sg,sgn,sgo,bl,bln,blo);


  // output result file
  btime = frtGetTime();
  if( FileOutput(nstep,time,NORMALOUT) !=1 ) return -1;
  m_FOtime = m_FOtime +frtGetTime() - btime;

  // print history
  PrintHistory(nstep);

  // barrier
  ParaMngr->Barrier(procGrp);
  TotalLoopTime += SklShowIntervalTime(&LoopPrevTime);

  // total calculation step
  calcStep = step;

  return 1;
}

