c
c<calculate velocity divergence with the fourth-order accuracy
c

      subroutine caldiv(ID,vsall,vc,ni,nj,nk,icherr,nstep,imon_t,
     1  dx,dy,dz,dt,time,uvwn,div,rlog,n,ftime)
c
      implicit none
      include 'sklparaf.h'

cc subroutine arguments
      integer ID
      integer vsall(3)
      integer vc
      integer ni,nj,nk
      integer icherr,nstep,imon_t
      real dx,dy,dz
      real dt,time
      real uvwn(3,1-vc:ni+vc,1-vc:nj+vc,1-vc:nk+vc)
      real div   (1-vc:ni+vc,1-vc:nj+vc,1-vc:nk+vc)
      real rlog(43)
      integer n
      real*8 ftime(10)

cc local variables 
      integer ierr
      integer i,j,k
      real sum,err,av
      real dxdtinv,dydtinv,dzdtinv
      real dun1,dvn1,dwn1
      real*8 sendbuf(2),recvbuf(2)
      real*8 frt_gettime
      real*8 frt_spantime
      real*8 tim
c
      dxdtinv=1.0d0/(dx*dt)
      dydtinv=1.0d0/(dy*dt)
      dzdtinv=1.0d0/(dz*dt)

c
!$OMP  PARALLEL DO
!$OMP$ DEFAULT(none)
!$OMP$ PRIVATE(i,j,k)
!$OMP$ PRIVATE(dun1,dvn1,dwn1)
!$OMP$ SHARED(un,vn,wn,div)
!$OMP$ SHARED(dxdtinv,dydtinv,dzdtinv)
      do k=1,nk
      do j=1,nj
      do i=1,ni
c
      dun1=-uvwn(1,i-1,j  ,k  )+uvwn(1,i,j,k)
      dvn1=-uvwn(2,i  ,j-1,k  )+uvwn(2,i,j,k)
      dwn1=-uvwn(3,i  ,j  ,k-1)+uvwn(3,i,j,k)
c
      div(i,j,k)=
     1 + dun1*dxdtinv
     2 + dvn1*dydtinv
     3 + dwn1*dzdtinv
c
c      if(i.eq.1 .and. j.eq.1 .and. k.eq.1 ) then
c        write(*,*) "div:",i,j,k,div(i,j,k)
c        write(*,*) "uvwn(1,i-1,j,k) :",uvwn(1,i-1,j,k)
c        write(*,*) "uvwn(1,i,j,k)   :",uvwn(1,i,j,k)
c        write(*,*) "uvwn(2,i,j-1,k) :",uvwn(2,i,j-1,k)
c        write(*,*) "uvwn(2,i,j,k)   :",uvwn(2,i,j,k)
c        write(*,*) "uvwn(3,i,j,k-1) :",uvwn(3,i,j,k-1)
c        write(*,*) "uvwn(3,i,j,k)   :",uvwn(3,i,j,k)
c      endif

      enddo
      enddo
      enddo
!$OMP END PARALLEL DO

c      call SklCommBndCell(div,vc,ierr)

c
c<display rms error of the velocity divergence
c
      if(icherr.eq.1.and.mod(nstep,imon_t).eq.0)then
      sum=0.0d0
      err=0.0d0

!$OMP  PARALLEL DO
!$OMP$ DEFAULT(none)
!$OMP$ PRIVATE(i,j,k)
!$OMP$ SHARED(div)
!$OMP$ REDUCTION(+:sum)
!$OMP$ REDUCTION(+:err)
      do k=1,nk
      do j=1,nj
      do i=1,ni
      sum=sum+div(i,j,k)
      err=err+div(i,j,k)**2
      enddo
      enddo
      enddo
!$OMP END PARALLEL DO

      sendbuf(1) = sum
      recvbuf(1) = sum
      sendbuf(2) = err
      recvbuf(2) = err

      tim=frt_gettime()
      call SklAllreduce(sendbuf,recvbuf,2,SKL_REAL8,SKL_SUM,0,ierr)
      ftime(6)=ftime(6)+frt_spantime(tim)

      sum = recvbuf(1)
      err = recvbuf(2)

      av=      sum/dble(vsall(1)*vsall(2)*vsall(3)) *dt
      err=sqrt(err/dble(vsall(1)*vsall(2)*vsall(3)))*dt

c      if( ID==0 ) then 
      write(*,'("Err_caldiv  ",20e20.10)')time,err,av
c      endif

      if( n==1 )then
      rlog(13)=time
      rlog(14)=err
      rlog(15)=av
      endif

      if( n==2 )then
      rlog(41)=time
      rlog(42)=err
      rlog(43)=av
      endif

      endif
c
      return
      end

c -------------------------------------------------------------

